library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

--! Packages from IP cores
use work.wishbone_pkg.ALL;
use work.i2c_slave_pkg.ALL;
use work.m25p32_pkg.ALL;
use work.multiboot_pkg.ALL;
use work.rtm_detector_pkg.ALL;

package image1_pkg is

   constant c_WB_CLK_PERIOD    : TIME    := 50 ns;
   constant c_RST_CLKS         : NATURAL := 256; --! @8ns

   constant c_NUM_MASTERS      : NATURAL := 1;
   constant c_NUM_SLAVES       : NATURAL := 3;

   constant c_MASTER_I2C_SLAVE : NATURAL := 0;
   constant c_SLAVE_I2C_SLAVE  : NATURAL := 0;
   constant c_SLAVE_MULTIBOOT  : NATURAL := 1;
   constant c_SLAVE_M25P32     : NATURAL := 2;

   constant c_ADDR_MULTIBOOT   : t_wishbone_address := X"00000200";
   constant c_ADDR_M25P32      : t_wishbone_address := X"00000080";
   constant c_ADDR_I2C_SLAVE   : t_wishbone_address := X"00000040";

   --! 64 words per page: 6 + 1 bits
   constant c_MASK_M25P32      : t_wishbone_address := X"FFFFFFE0";
   constant c_MASK_MULTIBOOT   : t_wishbone_address := X"FFFFFFC0";
   constant c_MASK_I2C_SLAVE   : t_wishbone_address := X"FFFFFFC0";

   constant c_addresses  : t_wishbone_address_array(c_NUM_SLAVES - 1 downto 0)
                                                         := (c_ADDR_MULTIBOOT,
                                                             c_ADDR_M25P32,
                                                             c_ADDR_I2C_SLAVE);

   constant c_masks      : t_wishbone_address_array(c_NUM_SLAVES - 1 downto 0)
                                                         := (c_MASK_MULTIBOOT,
                                                             c_MASK_M25P32,
                                                             c_MASK_I2C_SLAVE);

   component basic_trigger_top
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (clk_125m_i     : in  STD_LOGIC;
         led_pw_o       : out STD_LOGIC;
         led_err_o      : out STD_LOGIC;
         led_ttl_o      : out STD_LOGIC;

         fpga_o_en      : out STD_LOGIC;
         fpga_o_ttl_en  : out STD_LOGIC;
         fpga_o_inv_en  : out STD_LOGIC;
         fpga_o_blo_en  : out STD_LOGIC;

         level          : in  STD_LOGIC;
         switch_i       : in  STD_LOGIC;   --! General enable
         manual_rst_n_o : out STD_LOGIC;   --! It allows power sequencing of the
                                           --! 24V rail after a security given
                                           --! delay
         pulse_i_front  : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_front  : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);

         pulse_i_rear   : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_rear   : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);

         led_o_front    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         led_o_rear     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);

         --! This LED will show the status of the PLL
         led_link_up_o  : out STD_LOGIC;
         --! WR LEDs not to let them ON
         led_pps_o      : out STD_LOGIC;
         led_wr_ok_o    : out STD_LOGIC;

         inv_i          : in  STD_LOGIC_VECTOR(4 downto 1);
         inv_o          : out STD_LOGIC_VECTOR(4 downto 1));
   end component;


   component i2c_slave_top
   generic(g_WB_CLK_PERIOD : TIME := c_WB_CLK_PERIOD); -- Specify in ns
   port(sda_oen          : out STD_LOGIC;
        sda_i            : in  STD_LOGIC;
        sda_o            : out STD_LOGIC;
        scl_oen          : out STD_LOGIC;
        scl_i            : in  STD_LOGIC;
        scl_o            : out STD_LOGIC;

        wb_clk           : in  STD_LOGIC;
        wb_rst_i         : in  STD_LOGIC;

        wb_master_stb_o  : out STD_LOGIC;
        wb_master_cyc_o  : out STD_LOGIC;
        wb_master_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
        wb_master_we_o   : out STD_LOGIC;
        wb_master_data_i : in  STD_LOGIC_VECTOR(31 downto 0);
        wb_master_data_o : out STD_LOGIC_VECTOR(31 downto 0);
        wb_master_addr_o : out STD_LOGIC_VECTOR(15  downto 0);
        wb_master_ack_i  : in  STD_LOGIC;
        wb_master_rty_i  : in  STD_LOGIC;
        wb_master_err_i  : in  STD_LOGIC;

        wb_slave_stb_i   : in  STD_LOGIC;
        wb_slave_cyc_i   : in  STD_LOGIC;
        wb_slave_sel_i   : in  STD_LOGIC_VECTOR(3  downto 0);
        wb_slave_we_i    : in  STD_LOGIC;
        wb_slave_data_i  : in  STD_LOGIC_VECTOR(31 downto 0);
        wb_slave_data_o  : out STD_LOGIC_VECTOR(31 downto 0);
        wb_slave_addr_i  : in  STD_LOGIC_VECTOR(3  downto 0);
        wb_slave_ack_o   : out STD_LOGIC;
        wb_slave_rty_o   : out STD_LOGIC;
        wb_slave_err_o   : out STD_LOGIC;

        pf_wb_addr_o     : out STD_LOGIC;
        rd_done_o        : out STD_LOGIC;
        wr_done_o        : out STD_LOGIC;
        i2c_addr_i       : in  STD_LOGIC_VECTOR(6  downto 0));
   end component;

   component m25p32_top
   generic(g_WB_ADDR_LENGTH : NATURAL := c_WORDS_PER_PAGE_BITS + 1);
   port(wb_rst_i       : in  STD_LOGIC;
        wb_clk         : in  STD_LOGIC;

        wb_we_i        : in  STD_LOGIC;
        wb_stb_i       : in  STD_LOGIC;
        wb_cyc_i       : in  STD_LOGIC;
        wb_sel_i       : in  STD_LOGIC_VECTOR (3  downto 0);
        wb_data_i      : in  STD_LOGIC_VECTOR (31 downto 0);
        wb_data_o      : out STD_LOGIC_VECTOR (31 downto 0);
        wb_addr_i      : in  STD_LOGIC_VECTOR (g_WB_ADDR_LENGTH - 1 downto 0);
        wb_ack_o       : out STD_LOGIC;
        wb_rty_o       : out STD_LOGIC;
        wb_err_o       : out STD_LOGIC;

        miso_word_rcv  : out STD_LOGIC;
        op_finished_o  : out STD_LOGIC;

        prom_mosi_o    : out STD_LOGIC;
        prom_cclk_o    : out STD_LOGIC;
        prom_cs0_b_n_o : out STD_LOGIC;
        prom_din_i     : in  STD_LOGIC);
   end component;

   component multiboot_top
   generic(g_MBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_MBA_map_addr);
           g_GBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_GBA_map_addr);
           --! This is a vendor-dependant SPI opcode.
           --! Pleasem take a look to the flash memory manual and put here
           --! the op-code/instruction-code for a read operation over SPI
           --! i.e. for m25p32 memory that corresponds to X"03"
           --! Set it up accordingly.
           g_READ_SPI_OPCODE : STD_LOGIC_VECTOR(7 downto 0) := X"03");
   port(wb_rst_i   : in   STD_LOGIC;
        wb_clk     : in   STD_LOGIC;
        wb_we_i    : in   STD_LOGIC;
        wb_stb_i   : in   STD_LOGIC;
        wb_cyc_i   : in   STD_LOGIC;
        wb_sel_i   : in   STD_LOGIC_VECTOR (3  downto 0);
        wb_data_i  : in   STD_LOGIC_VECTOR (31 downto 0);
        wb_data_o  : out  STD_LOGIC_VECTOR (31 downto 0);
        wb_addr_i  : in   STD_LOGIC_VECTOR (3  downto 0);
        wb_ack_o   : out  STD_LOGIC;
        wb_rty_o   : out  STD_LOGIC;
        wb_err_o   : out  STD_LOGIC);
   end component;

   component rtm_detector
   generic(g_identifier_RTMM : t_RTMM := RTMM_V1;
           g_identifier_RTMP : t_RTMP := RTMP_BLOCKING_V1);
   port (RTMM_i     : in STD_LOGIC_VECTOR(2 downto 0);
         RTMP_i     : in STD_LOGIC_VECTOR(2 downto 0);
         ok_RTMM_o  : out STD_LOGIC;
         ok_RTMP_o  : out STD_LOGIC);
   end component;

end image1_pkg;

package body image1_pkg is
end image1_pkg;
